@@ -78,7 +78,7 @@ class Historical:
                                                                                           "select_clause": select_clause, "where_clause": where_clause,
                                                                                           "maxhits_clause": maxhits, "group_by_clause": group_by, "order_by_clause": order_by }))
     
-    def get_alerts_stats(self, epoch_begin, epoch_end):
+    def get_alerts_stats(self, epoch_begin, epoch_end, host=None):
         """
         Return flow alerts stats
         
@@ -86,10 +86,17 @@ class Historical:
         :type epoch_begin: int
         :param epoch_end: End of the time interval (epoch)
         :type epoch_end: int
+        :param host: Host IP address (optional)
+        :type host: string
         :return: Flow alert stats
         :rtype: object
         """
-        return(self.ntopng_obj.request(self.rest_v2_url + "/get/alert/top.lua", { "ifid": self.ifid, "epoch_begin": epoch_begin, "epoch_end": epoch_end }))
+        get_data = { "ifid": self.ifid, "epoch_begin": epoch_begin, "epoch_end": epoch_end }
+        
+        if host is not None:
+            get_data["ip"] = host + ";eq"
+        
+        return(self.ntopng_obj.request(self.rest_v2_url + "/get/alert/top.lua", get_data))
 
     def get_flow_alerts_stats(self, epoch_begin, epoch_end):
         """
@@ -342,6 +349,24 @@ class Historical:
         rsp = self.ntopng_obj.post_request(self.rest_v2_url + "/get/timeseries/ts.lua", { "ts_schema": ts_schema, "ts_query": ts_query, "epoch_begin": epoch_begin, "epoch_end": epoch_end })
         return self.timeseries_to_pandas(rsp)
 
+    def get_timeseries_stats(self, ts_schema, ts_query, epoch_begin, epoch_end):
+        """
+        Return stats from timeseries
+        
+        :param ts_schema: The timeseries schema (e.g. 'host:traffic')
+        :type ts_schema: string
+        :param ts_query: The timeseries query (e.g. 'ifid:0,host:10.0.0.1')
+        :type ts_query: string
+        :param epoch_begin: Start of the time interval (epoch)
+        :type epoch_begin: int
+        :param epoch_end: End of the time interval (epoch)
+        :type epoch_end: int
+        :return: Timeseries data
+        :rtype: object (pandas DataFrame)
+        """
+        rsp = self.ntopng_obj.post_request(self.rest_v2_url + "/get/timeseries/ts.lua", { "ts_schema": ts_schema, "ts_query": ts_query, "epoch_begin": epoch_begin, "epoch_end": epoch_end })
+        return rsp['statistics']
+
     def get_timeseries_metadata(self):
         """
         Return timeseries metadata (list all available timeseries)
@@ -368,6 +393,23 @@ class Historical:
         """
         return(self.get_timeseries(ts_schema, "ifid:"+str(self.ifid)+",host:"+host_ip, epoch_begin, epoch_end))
 
+    def get_host_timeseries_stats(self, host_ip, ts_schema, epoch_begin, epoch_end):
+        """
+        Return timeseries statistics
+        
+        :param host_ip: The host IP
+        :type host: string
+        :param ts_schema: The timeseries schema
+        :type ts_schema: string
+        :param epoch_begin: Start of the time interval (epoch)
+        :type epoch_begin: int
+        :param epoch_end: End of the time interval (epoch)
+        :type epoch_end: int
+        :return: Timeseries data
+        :rtype: object (pandas DataFrame)
+        """
+        return(self.get_timeseries_stats(ts_schema, "ifid:"+str(self.ifid)+",host:"+host_ip, epoch_begin, epoch_end))
+
     def get_interface_timeseries(self, ts_schema, epoch_begin, epoch_end):
         """
         Return timeseries data in a pandas DataFrame for a specified interface
@@ -383,6 +425,21 @@ class Historical:
         """
         return(self.get_timeseries(ts_schema, "ifid:"+str(self.ifid), epoch_begin, epoch_end))
 
+    def get_interface_timeseries_stats(self, ts_schema, epoch_begin, epoch_end):
+        """
+        Return timeseries statistics
+        
+        :param ts_schema: The timeseries schema
+        :type ts_schema: string
+        :param epoch_begin: Start of the time interval (epoch)
+        :type epoch_begin: int
+        :param epoch_end: End of the time interval (epoch)
+        :type epoch_end: int
+        :return: Timeseries data
+        :rtype: object (pandas DataFrame)
+        """
+        return(self.get_timeseries_stats(ts_schema, "ifid:"+str(self.ifid), epoch_begin, epoch_end))
+
     def get_flows(self, epoch_begin, epoch_end, select_clause, where_clause, maxhits, group_by, order_by):
         """
         Run queries on the historical flows database (ClickHouse)
@@ -423,6 +480,46 @@ class Historical:
         """
         return(self.ntopng_obj.request(self.rest_pro_v2_url + "/get/db/topk_flows.lua", {"ifid": self.ifid, "begin_time_clause": epoch_begin, "end_time_clause": epoch_end, "maxhits_clause": max_hits, "where_clause": where_clause }))
 
+    def get_top_conversations(self, epoch_begin, epoch_end, host=None):
+        """
+        Return Top Conversations
+        
+        :param epoch_begin: Start of the time interval (epoch)
+        :type epoch_begin: int
+        :param epoch_end: End of the time interval (epoch)
+        :type epoch_end: int
+        :param host: Host IP address (optional)
+        :type host: string
+        :return: Top conversations
+        :rtype: object
+        """
+        get_data = { "ifid": self.ifid, "epoch_begin": epoch_begin, "epoch_end": epoch_end, "query_preset": "top_conversations", "order": "DESC", "start": 0, "length": 10, "sort": "bytes", "visible_columns": "cli_ip,srv_ip,bytes" }
+        
+        if host is not None:
+            get_data["ip"] = host + ";eq"
+        
+        res = self.ntopng_obj.request(self.rest_pro_v2_url + "/get/db/historical_db_search.lua", get_data)
+        return(res["records"])
+
+    def get_host_top_protocols(self, host, epoch_begin, epoch_end):
+        """
+        Return Top protocols for an host
+
+        :param host: Host IP address
+        :type host: string
+        :param epoch_begin: Start of the time interval (epoch)
+        :type epoch_begin: int
+        :param epoch_end: End of the time interval (epoch)
+        :type epoch_end: int
+        :return: Top protocols
+        :rtype: object
+        """
+
+        get_data = {  "ts_query": "ifid:" + self.ifid + ",host:" + host, "detail_view": "top_protocols", "epoch_begin": epoch_begin, "epoch_end": epoch_end, "new_charts": "true" }
+
+        res = self.ntopng_obj.request(self.rest_pro_v2_url + "/get/host/top/ts_stats.lua", get_data)
+        return(res)
+
     def self_test(self, host):
         try:
             epoch_end   = int(time.time())
@@ -454,15 +551,23 @@ class Historical:
             print(self.get_timeseries("host:traffic", "ifid:"+str(self.ifid)+",host:"+host, epoch_begin, epoch_end))
             print("Interface traffic timeseries ----------------------------")
             print(self.get_interface_timeseries("iface:traffic_rxtx", epoch_begin, epoch_end))
+            print("Host traffic timeseries ----------------------------")
+            print(self.get_host_timeseries(host, "host:traffic", epoch_begin, epoch_end))
             print("Interface score timeseries ----------------------------")
             print(self.get_interface_timeseries("iface:score", epoch_begin, epoch_end))
             print("Host flows ----------------------------")
             select_clause = "IPV4_SRC_ADDR,IPV4_DST_ADDR,PROTOCOL,IP_SRC_PORT,IP_DST_PORT,L7_PROTO,L7_PROTO_MASTER"
             where_clause  = "(IP_PROTOCOL_VERSION=4) AND IPV4_SRC_ADDR=(\""+host+"\") AND (PROTOCOL=6) "
             maxhits       = 10 # 10 records max
-            print(self.get_flows( epoch_begin, epoch_end, select_clause, where_clause, maxhits, '', ''))
-            print("----------------------------")
+            print(self.get_flows(epoch_begin, epoch_end, select_clause, where_clause, maxhits, '', ''))
+            print("Top K flows ----------------------------")
             print(self.get_topk_flows(epoch_begin, epoch_end, maxhits, None))
+            print("Host alerts ----------------------------")
+            print(self.get_alerts_stats(epoch_begin, epoch_end, host))
+            print("Host Top conversations ----------------------------")
+            print(self.get_top_conversations(epoch_begin, epoch_end, host))
+            print("Host Top protocols ----------------------------")
+            print(self.get_host_top_protocols(host, epoch_begin, epoch_end))
             print("----------------------------")
         except:
             raise ValueError("Invalid interface ID, host or parameters specified")
